﻿using System;
using System.Collections;
using System.Configuration;
using System.Data;
using System.Linq;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Xml.Linq;
using DivulgacionCientifica.CapaNegocio;
using System.Data.SqlClient;
using System.Collections.Generic;
using System.IO;
using System.Drawing;
using System.Drawing.Imaging;

namespace DivulgacionCientifica
{
    public partial class EditarPublicAdmin : System.Web.UI.Page
    {
        string idCurrentUser = HttpContext.Current.User.Identity.Name;
        Publicacion pub;

        protected void Page_Load(object sender, EventArgs e)
        {
            LabelError.Text = "";
            LabelCorrecto.Text = "";
            try
            {
                if ((Request.Params["id"] == null) || (!Usuario.esAdministrador(idCurrentUser)))
                {
                    Response.Redirect("Inicio.aspx");
                }
                else
                {

                    string stringIdNOt = Request.Params["id"];
                    int id = Int32.Parse(stringIdNOt);
                    pub = new Publicacion(Int32.Parse(Request.Params["id"]));

                    if (!IsPostBack) //Solo lo hace al principio
                    {
                        Auxiliares.cargarTemas(DropDownListTemas);
                        TextBoxTitulo.Text = pub.getTitulo();
                        if (pub.getContenido().Contains("<") && pub.getContenido().Contains(">"))
                        {
                            FreeTextBox.Text = pub.getContenido();
                            LinkButtonEditor.Text = "Usar el editor básico";
                            PanelAvanzado.Visible = true;
                            PanelBasico.Visible = false;
                        }
                        else
                        {
                            TextBoxDescripcion.Text = pub.getContenido();
                            LinkButtonEditor.Text = "Usar el editor avanzado";
                            PanelAvanzado.Visible = false;
                            PanelBasico.Visible = true;
                        }
                        cargarTemasDeLaPublic();
                        cargarNombresArchivos();
                        cargarReferencias();
                        cargarCategoria();
                        cargarFechaCaducidad();
                    }
                }
                
            }
            catch (NullReferenceException)
            {
                //Se produce cuando la noticia no existe ya que se accede a un campo nulo
                Response.Redirect("Inicio.aspx");
            }
        }

        protected void cargarNombresArchivos()
        {
            List<Archivo> archs = pub.getArchivos(false);
            foreach (Archivo arch in archs)
            {
                string nombre = arch.getRuta().Substring(arch.getRuta().LastIndexOf(Path.DirectorySeparatorChar) + 1);
                DropDownListTemp.Items.Add(nombre);
            }
        }

        private void cargarTemasDeLaPublic()
        {
            List<Tema> temas = pub.getTemas();
            foreach (Tema tema in temas)
            {
                DropDownListTemas0.Items.Add(tema.getNombre());
            }
        }

        private void cargarReferencias()
        {
            List<Referencia> referencias = pub.getReferencias(true);
            foreach (Referencia referencia in referencias)
            {
                DropDownListRef.Items.Add(referencia.getReferencia());
            }
        }

        private void cargarCategoria()
        {
            RadioButtonList1.Items.FindByText(pub.getCategoria()).Selected = true;   
        }

        private void cargarFechaCaducidad()
        {
            if (pub.getCaducidad().Equals("NULL"))
            {
                //No selecciono ninguna fecha
            }
            else
            {
                DateTime fechaCad = Auxiliares.StringAFecha(pub.getCaducidad());
                Calendar1.SelectedDate = fechaCad;
            }
        }

        protected void btEliminar_Click(object sender, EventArgs e)
        {
            try
            {
                if ((DropDownListTemp.Items.Count > 0) && (DropDownListTemp.SelectedItem != null) && (!DropDownListTemp.SelectedItem.Text.Equals("")))
                {
                    string ruta = Estatica.getRutaProyecto + "Archivos" + Path.DirectorySeparatorChar + DropDownListTemp.SelectedItem.Text;
                    string nombre = ruta.Substring(ruta.LastIndexOf(Path.DirectorySeparatorChar) + 1);
                    if (nombre.Contains("_"))
                    {
                        string idArch = nombre.Substring(0, nombre.IndexOf("_"));
                        if (File.Exists(ruta))
                        {
                            Archivo ar = new Archivo(Int32.Parse(idArch));
                            ar.eliminar();
                        }
                        DropDownListTemp.Items.Remove(DropDownListTemp.SelectedItem.Text);
                    }
                    else
                    {
                        string archivo = Estatica.getRutaProyecto + "temp" + Path.DirectorySeparatorChar
                            + idCurrentUser + Path.DirectorySeparatorChar + nombre;
                        if (File.Exists(archivo))
                        {
                            File.Delete(archivo);
                        }
                        DropDownListTemp.Items.Remove(DropDownListTemp.SelectedItem);
                    }
                }
            }
            catch (IOException)
            {
                LabelError.Text = "ERROR. No se ha podido eliminar el archivo";
            }
            catch(SqlException)
            {
                LabelError.Text = "ERROR. No se ha podido eliminar el archivo";
            }
        }

        protected void btAnadirTema_Click(object sender, EventArgs e)
        {
            try
            {
                if ((DropDownListTemas.Items.Count > 0) && (DropDownListTemas.SelectedItem != null))
                {
                    if ((DropDownListTemas0.Items.FindByText(DropDownListTemas.SelectedItem.Text)) == null)
                    {
                        pub.anadirTema(new Tema(DropDownListTemas.SelectedItem.Text, ""));
                        DropDownListTemas0.Items.Add(DropDownListTemas.SelectedItem.Text);
                    }
                    else
                    {
                        LabelError.Text = "La publicación ya esta asociada a este tema.";
                    }
                }
            }
            catch (SqlException)
            {
                LabelError.Text = "ERROR. No se ha podido relacionar la publicación con dicho tema.";
            }
        }

        protected void btEiminarTema_Click(object sender, EventArgs e)
        {
            try
            {
                if (DropDownListTemas0.Items.Count <= 1)
                {
                    LabelError.Text = "No puede eliminar dicho tema ya que la publicación debe estar asociada al menos a un tema";
                }
                else if (DropDownListTemas0.SelectedItem != null)
                {
                    pub.eliminarTema(new Tema(DropDownListTemas0.SelectedItem.Text, ""));
                    DropDownListTemas0.Items.Remove(DropDownListTemas0.SelectedItem.Text);
                }
            }
            catch (SqlException)
            {
                LabelError.Text = "ERROR. No se ha podido eliminar la asociación de esta publicacion a dicho tema.";
            }
        }

        protected void btAnadirRef_Click(object sender, EventArgs e)
        {
            try
            {
                if (TextBoxRef.Text.Equals(""))
                {
                    LabelError.Text = "Debe escribir la referencia.";
                }
                else
                {
                    Referencia refe = new Referencia(TextBoxRef.Text, new Publicacion(pub.getID()));
                    refe.almacenarAdmin();
                    DropDownListRef.Items.Add(TextBoxRef.Text);
                }
            }
            catch (SqlException)
            {
                LabelError.Text = "ERROR. No se ha podido almacenar la referencia.";
            }
        }

        protected void btEliminarRef_Click(object sender, EventArgs e)
        {
            try
            {
                if ((DropDownListRef.Items.Count > 0) && (DropDownListRef.SelectedItem != null))
                {
                    Referencia refe = new Referencia(DropDownListRef.SelectedItem.Text, new Publicacion(pub.getID()));
                    refe.eliminar();
                    DropDownListRef.Items.Remove(DropDownListRef.SelectedItem.Text);
                }
                else
                {
                    LabelError.Text = "Debe seleccionar la referencia.";
                }
            }
            catch (SqlException)
            {
                LabelError.Text = "ERROR. No se ha podido eliminar la referencia.";
            }
        }

        protected void LinkButtonEditor_Click(object sender, EventArgs e)
        {
            if (LinkButtonEditor.Text.Equals("Usar el editor avanzado"))
            {
                PanelBasico.Visible = false;
                PanelAvanzado.Visible = true;
                LinkButtonEditor.Text = "Usar el editor básico";
                FreeTextBox.Text = TextBoxDescripcion.Text;
            }
            else
            {
                PanelBasico.Visible = true;
                PanelAvanzado.Visible = false;
                TextBoxDescripcion.Text = FreeTextBox.Text;
                LinkButtonEditor.Text = "Usar el editor avanzado";
            }
        }

        protected void btModCategoria_Click(object sender, EventArgs e)
        {
            try
            {
                pub.modificarCategoria(RadioButtonList1.SelectedItem.Text);
            }
            catch (SqlException)
            {
                LabelError.Text = "ERROR. No se ha podido modificar la categoría.";
            }
        }

        protected void btModTitulo_Click(object sender, EventArgs e)
        {
            try
            {
                pub.modificarTitulo(TextBoxTitulo.Text);
            }
            catch (SqlException)
            {
                LabelError.Text = "ERROR. No se ha podido modificar el título.";
            }
        }

        protected void btModContenido_Click(object sender, EventArgs e)
        {
            try
            {
                if (PanelAvanzado.Visible)
                {
                    pub.modificarContenido(FreeTextBox.Text);
                }
                else
                {
                    pub.modificarContenido(TextBoxDescripcion.Text);
                }
            }
            catch (SqlException)
            {
                LabelError.Text = "ERROR. No se ha podido modificar el contenido";
            }
        }

        protected void btAnadirArch_Click(object sender, EventArgs e)
        {
            /*
             * Debo primero guardar los archivos en la BD y luego salvarlos como en el metodo comentado.
             */
            try
            {
                string rutaTemp = Estatica.getRutaProyecto + "Archivos";
                if (!Directory.Exists(rutaTemp))
                {
                    Directory.CreateDirectory(rutaTemp);
                }

                if (FileUploadArch.HasFile && FileUploadArch.PostedFile.ContentLength < 10485760)
                {

                    if (FileUploadArch.FileName.ToLower().EndsWith(".jpg") ||
                        FileUploadArch.FileName.ToLower().EndsWith(".jpeg") ||
                        FileUploadArch.FileName.ToLower().EndsWith(".bmp") ||
                        FileUploadArch.FileName.ToLower().EndsWith(".gif") ||
                        FileUploadArch.FileName.ToLower().EndsWith(".tiff") ||
                        FileUploadArch.FileName.ToLower().EndsWith(".tif") ||
                        FileUploadArch.FileName.ToLower().EndsWith(".png"))
                    {
                        //Cambio los caracteres que al subir al servidor da error la URL
                        string nombre = FileUploadArch.FileName;
                        nombre = nombre.Replace('%', '-');
                        nombre = nombre.Replace('#', '-');
                        nombre = nombre.Replace('_', '-');
                        nombre = nombre.Replace(" ", "");

                        if (CheckBoxAsoc.Checked)
                        {
                            Archivo arch = new Archivo(rutaTemp + Path.DirectorySeparatorChar + nombre,
                                new Usuario(idCurrentUser), new Publicacion(pub.getID()));
                            string nombreFinal = arch.almacenarBD(true);
                            Archivo arch2 = new Archivo(rutaTemp + Path.DirectorySeparatorChar + "vp_" + nombre,
                                new Usuario(idCurrentUser), new Publicacion(pub.getID()));
                            string nombreFinal2 = arch2.almacenarBD(true);

                            string rrr = rutaTemp + Path.DirectorySeparatorChar + "red" + nombreFinal;
                            FileUploadArch.SaveAs(rrr);

                            Auxiliares.redimensionar(rutaTemp + Path.DirectorySeparatorChar + "red" + nombreFinal,
                                                     rutaTemp + Path.DirectorySeparatorChar + nombreFinal, 800, 800);
                            Auxiliares.redimensionar(rutaTemp + Path.DirectorySeparatorChar + "red" + nombreFinal,
                                                     rutaTemp + Path.DirectorySeparatorChar + nombreFinal2, 175, 175);
                            File.Delete(rutaTemp + Path.DirectorySeparatorChar + "red" + nombreFinal);

                            //Si el archivo con ese nombre existia en la carpeta se sobreescribe y en ese caso ya estaba en el 
                            //dropdownlist asi que no lo vuelvo a agregar
                            if (DropDownListTemp.Items.FindByText(nombreFinal) == null)
                            {
                                DropDownListTemp.Items.Add(nombreFinal);
                            }
                            pub.validarArchivos(); //pq cuando se insertan no estan validados pero como este es administrador... se validan automaticamente
                        }
                        else
                        {
                            FileUploadArch.SaveAs(rutaTemp + Path.DirectorySeparatorChar + "red" + nombre);
                            Auxiliares.redimensionar(rutaTemp + Path.DirectorySeparatorChar + "red" + nombre,
                                rutaTemp + Path.DirectorySeparatorChar + nombre, 800, 800);
                            File.Delete(rutaTemp + Path.DirectorySeparatorChar + "red" + nombre);

                            Archivo arch = new Archivo(rutaTemp + Path.DirectorySeparatorChar + nombre, new Usuario(idCurrentUser), new Publicacion(-1));
                            string nombreFinal = arch.almacenar();

                            //Si el archivo con ese nombre existia en la carpeta se sobreescribe y en ese caso ya estaba en el 
                            //dropdownlist asi que no lo vuelvo a agregar
                            if (DropDownListTemp.Items.FindByText(Estatica.getUrlProyecto + "Archivos/" + nombreFinal) == null)
                            {
                                DropDownListTemp.Items.Add(Estatica.getUrlProyecto + "Archivos/" + nombreFinal);
                            }
                        }
                    }
                    else if (FileUploadArch.FileName.ToLower().EndsWith(".wmv") ||
                        FileUploadArch.FileName.ToLower().EndsWith(".avi") ||
                        FileUploadArch.FileName.ToLower().EndsWith(".mov") ||
                        FileUploadArch.FileName.ToLower().EndsWith(".mpg") ||
                        FileUploadArch.FileName.ToLower().EndsWith(".mpeg") ||
                        FileUploadArch.FileName.ToLower().EndsWith(".mid") ||
                        FileUploadArch.FileName.ToLower().EndsWith(".midi") ||
                        FileUploadArch.FileName.ToLower().EndsWith(".mp3"))
                    {
                        string nombre = FileUploadArch.FileName;
                        nombre = nombre.Replace('%', '-');
                        nombre = nombre.Replace('#', '-');
                        nombre = nombre.Replace('_', '-');
                        nombre = nombre.Replace(" ", "");

                        //Si el archivo no debe ir asociado a una noticia lo reconozco pq empieza por a_
                        if (CheckBoxAsoc.Checked)
                        {
                            Archivo arch = new Archivo(rutaTemp + Path.DirectorySeparatorChar + nombre, new Usuario(idCurrentUser), new Publicacion(pub.getID()));
                            string nombreFinal = arch.almacenarBD(true);
                            FileUploadArch.SaveAs(rutaTemp + Path.DirectorySeparatorChar + nombreFinal);
                            if (DropDownListTemp.Items.FindByText(nombreFinal) == null)
                                DropDownListTemp.Items.Add(nombreFinal);
                        }
                        else
                        {
                            Archivo arch = new Archivo(rutaTemp + Path.DirectorySeparatorChar + nombre, new Usuario(idCurrentUser), new Publicacion(-1));
                            string nombreFinal = arch.almacenarBD(true);
                            FileUploadArch.SaveAs(rutaTemp + Path.DirectorySeparatorChar + nombreFinal);

                            if (DropDownListTemp.Items.FindByText(Estatica.getUrlProyecto + "Archivos/" + nombreFinal) == null)
                                DropDownListTemp.Items.Add(Estatica.getUrlProyecto + "Archivos/" + nombreFinal);
                        }
                    }
                    else
                    {
                        LabelError.Text = "ERROR. Tipo de archivo no soportado";
                    }
                }
                else if (FileUploadArch.PostedFile.ContentLength > 10485760)
                {
                    LabelError.Text = "ERROR. El archivo no puede tener un tamaño superior a 10Mb";
                }
                else
                {
                    LabelError.Text = "ERROR. El archivo indicado no existe o no es accesible.";
                }
            }
            catch (IOException)
            {
                LabelError.Text = "Se ha producido un error al intentar guardar el archivo.";
            }
        }
       
        /*
        protected void btAnadirArch_Click2(object sender, EventArgs e)
        {
            string nombre = "";
            int idNotic = -1;
            try
            {
                if (FileUploadArch.HasFile)
                {
                    if (CheckBoxAsoc.Checked)
                    {
                        idNotic = publicac.getID();
                        Archivo ar = new Archivo(FileUploadArch.FileName, currentUser, idNotic);
                        nombre = ar.almacenarBD(true);
                        FileUploadArch.SaveAs(Estatica.getRutaProyecto + "Archivos" + Path.DirectorySeparatorChar + nombre);
                        DropDownListTemp.Items.Add(nombre);
                    }
                    else
                    {
                        idNotic = -1;
                        Archivo ar = new Archivo(FileUploadArch.FileName, currentUser, idNotic);
                        nombre = ar.almacenarBD(true);
                        FileUploadArch.SaveAs(Estatica.getRutaProyecto + "Archivos" + Path.DirectorySeparatorChar + nombre);
                        DropDownListTemp.Items.Add(Estatica.getUrlProyecto+"Archivos/"+nombre);
                    }
                }
                else
                {
                    LabelError.Text = "Debe seleccionar un archivo.";
                }
            }
            catch (IOException)
            {
                Archivo ar = new Archivo(Estatica.getRutaProyecto + "Archivos" + Path.DirectorySeparatorChar + nombre, currentUser, idNotic);
                ar.eliminar();
                LabelError.Text = "ERROR. No se ha podido guardar el archivo.";
            }
            catch (SqlException)
            {
                LabelError.Text = "ERROR. No se ha podido guardar el archivo.";
            }
        }
        */

        protected void btModCaducidad_Click(object sender, EventArgs e)
        { 
            try
            {
                    string fe;
                    if ((Calendar1.SelectedDate == null) || (Calendar1.SelectedDate.CompareTo(DateTime.Now) < 0))
                    {
                        fe = "NULL";
                    }
                    else
                    {
                        fe = "'" + Calendar1.SelectedDate.Year.ToString() + "-" + Calendar1.SelectedDate.Month.ToString() + "-" +
                        Calendar1.SelectedDate.Day.ToString() + " " + Calendar1.SelectedDate.Hour.ToString() +
                        ":" + Calendar1.SelectedDate.Minute.ToString() + ":" + Calendar1.SelectedDate.Second.ToString() + "'";
                    }
                    pub.modificarCaducidad(fe);
            }
            catch (SqlException)
            {
                LabelError.Text = "ERROR. No se ha podido modificar la fecha de caducidad.";
            }
        }

        protected void btUsar_Click(object sender, EventArgs e)
        {

            if (TextBoxDescripcion.Visible)
            {
                TextBoxDescripcion.Text = TextBoxDescripcion.Text + DropDownListTemp.SelectedItem.Text;
            }
            else
            {
                FreeTextBox.Text = FreeTextBox.Text + DropDownListTemp.SelectedItem.Text;
            }
        }

        

    }
}
