﻿using System;
using System.Collections;
using System.Configuration;
using System.Data;
using System.Linq;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Xml.Linq;
using System.Collections.Generic;
using System.IO;
using System.Data.SqlClient;
using System.Drawing;
using System.Drawing.Imaging;

namespace DivulgacionCientifica.CapaNegocio
{
    public partial class EditarPublicUsu : System.Web.UI.Page
    {
        string idCurrentUser = HttpContext.Current.User.Identity.Name;
        Publicacion pub;

        protected void Page_Load(object sender, EventArgs e)
        {
            LabelError.Text = "";
            LabelCorrecto.Text = "";
            try
            {
                if (Request.Params["id"] == null)
                {
                    Response.Redirect("Inicio.aspx");
                }
                else
                {
                    pub = new Publicacion(Int32.Parse(Request.Params["id"]));

                    if (pub.getCreador().getId().Equals(idCurrentUser))
                    {
                        if (!IsPostBack) //Solo lo hace al principio
                        {
                            TextBoxTitulo.Text = pub.getTitulo();
                            if (pub.getContenido().Contains("<") && pub.getContenido().Contains(">"))
                            {
                                FreeTextBox.Text = pub.getContenido();
                                LinkButtonEditor.Text = "Usar el editor básico";
                                PanelAvanzado.Visible = true;
                                PanelBasico.Visible = false;
                            }
                            else
                            {
                                TextBoxDescripcion.Text = pub.getContenido();
                                PanelAvanzado.Visible = false;
                                PanelBasico.Visible = true;
                            }
                            cargarNombresArchivos();
                            cargarReferencias();
                            cargarCategoria();
                            cargarFechaCaducidad();
                        }
                    }
                    else
                    {
                        Response.Redirect("Inicio.aspx");
                    }
                }
            }
            catch (NullReferenceException)
            {
                //Se produce cuando la publicación no existe ya que se accede a un campo nulo
                Response.Redirect("Inicio.aspx");
            }
        }

        protected void cargarNombresArchivos()
        {
            List<Archivo> archs = pub.getArchivos(false);
            foreach (Archivo arch in archs)
            {
                string nombre = arch.getRuta().Substring(arch.getRuta().LastIndexOf(Path.DirectorySeparatorChar) + 1);
                DropDownListTemp.Items.Add(nombre);
            }
        }

        private void cargarReferencias()
        {
            List<Referencia> referencias = pub.getReferencias(true);
            foreach (Referencia referencia in referencias)
            {
                DropDownListRef.Items.Add(referencia.getReferencia());
            }
        }

        private void cargarCategoria()
        {
            RadioButtonList1.Items.FindByText(pub.getCategoria()).Selected = true;
        }

        private void cargarFechaCaducidad()
        {
            if (pub.getCaducidad().Equals("NULL"))
            {
                //No selecciono ninguna fecha
            }
            else
            {
                DateTime fechaCad = Auxiliares.StringAFecha(pub.getCaducidad());
                Calendar1.SelectedDate = fechaCad;
            }
        }

        protected void btAceptar_Click(object sender, EventArgs e)
        {
            string fe;
            if (Calendar1.SelectedDate.CompareTo(DateTime.Now) < 0){ fe = "NULL"; }
            else
            {
                fe = "'" + Calendar1.SelectedDate.Year.ToString() + "-" + Calendar1.SelectedDate.Month.ToString() + "-" +
                Calendar1.SelectedDate.Day.ToString() + " " + Calendar1.SelectedDate.Hour.ToString() +
                ":" + Calendar1.SelectedDate.Minute.ToString() + ":" + Calendar1.SelectedDate.Second.ToString() + "'";
            }
            string contenido;
            if (PanelBasico.Visible)
            {
                contenido = TextBoxDescripcion.Text;
            }
            else
            {
                contenido = FreeTextBox.Text;
            }

            if (TextBoxTitulo.Text.Equals("") ||
                    TextBoxTitulo.Text.Contains('\\') || TextBoxTitulo.Text.Contains('/') ||
                    TextBoxTitulo.Text.Contains(':') || TextBoxTitulo.Text.Contains('*') ||
                    TextBoxTitulo.Text.Contains('"') ||
                    TextBoxTitulo.Text.Contains('<') || TextBoxTitulo.Text.Contains('>') ||
                    TextBoxTitulo.Text.Contains('|') || (TextBoxTitulo.Text.Length > 45))
            {
                    LabelCorrecto.Text = "";
                    LabelError.Text = "El titulo no puede ser vacio, ni exceder de 45 caracteres, ni contener los caracteres \\ / : * \" < > |";
            }
            else if ((PanelAvanzado.Visible && FreeTextBox.Text.Length > 60000) || (PanelBasico.Visible && TextBoxDescripcion.Text.Length > 60000))
            {
                LabelCorrecto.Text = "";
                LabelError.Text = "El contenido no puede exceder de 60000 caracteres.";
            }
            else
            {
                try
                {
                    EditPublic mn = new EditPublic(pub.getID(), TextBoxTitulo.Text, contenido, RadioButtonList1.SelectedItem.Text, fe, TextBoxMotivo.Text, DateTime.Now);
                    mn.almacenar();
                    LabelCorrecto.Text = "La petición de modificación de ha sido almacenada con ÉXITO, recuerde que deberá ser validada antes de que sea visible";
                }
                catch (SqlException)
                {
                    LabelError.Text = "ERROR. No se ha podido almacenar la petición de modificación";
                }

                try
                {
                    guardarReferencias(pub.getID());
                }
                catch (SqlException)
                {
                    LabelError.Text += " pero no se han podido almacenar las referencias.";
                }

                try
                {
                    guardarArchivos(pub.getID());
                }
                catch (SqlException)
                {
                    LabelCorrecto.Text = "";
                    LabelError.Text += " Puede que alguno de sus archivos adjuntos no se haya guardado (BD)";
                }
                catch (IOException)
                {
                    LabelCorrecto.Text = "";
                    LabelError.Text += " Puede que alguno de sus archivos adjuntos no se haya guardado (IO)";
                }   
            }
        }

        protected void LinkButtonEditor_Click(object sender, EventArgs e)
        {
            if (LinkButtonEditor.Text.Equals("Usar el editor avanzado"))
            {
                PanelBasico.Visible = false;
                PanelAvanzado.Visible = true;
                LinkButtonEditor.Text = "Usar el editor básico";
                FreeTextBox.Text = TextBoxDescripcion.Text;
            }
            else
            {
                PanelBasico.Visible = true;
                PanelAvanzado.Visible = false;
                TextBoxDescripcion.Text = FreeTextBox.Text;
                LinkButtonEditor.Text = "Usar el editor avanzado";
            }
        }

        protected void btEliminar_Click(object sender, EventArgs e)
        {
            try
            {
                if ((DropDownListTemp.Items.Count > 0) && (DropDownListTemp.SelectedItem != null) && (!DropDownListTemp.SelectedItem.Text.Equals("")))
                {
                    string ruta = Estatica.getRutaProyecto + "Archivos" + Path.DirectorySeparatorChar + DropDownListTemp.SelectedItem.Text;
                    string nombre = ruta.Substring(ruta.LastIndexOf(Path.DirectorySeparatorChar) + 1);
                    if (nombre.Contains("_"))
                    {
                        string idArch = nombre.Substring(0, nombre.IndexOf("_"));
                        if (File.Exists(ruta))
                        {
                            Archivo ar = new Archivo(Int32.Parse(idArch));
                            ar.eliminar();
                        }
                        DropDownListTemp.Items.Remove(DropDownListTemp.SelectedItem.Text);
                        LabelCorrecto.Text = "El archivo ha sido eliminado correctamente.";
                    }
                    else
                    {
                        //Es que no estaba guardado, lo ha añadido y lo ha borrado justo después
                        string archivo = Estatica.getRutaProyecto + "temp" + Path.DirectorySeparatorChar + idCurrentUser + Path.DirectorySeparatorChar + nombre;
                        if (File.Exists(archivo))
                        {
                            File.Delete(archivo);
                        }
                        string archivoVp = Estatica.getRutaProyecto + "temp" + Path.DirectorySeparatorChar + idCurrentUser + Path.DirectorySeparatorChar +"vp_"+ nombre;
                        if (File.Exists(archivoVp))
                        {
                            File.Delete(archivoVp);
                        }
                        DropDownListTemp.Items.Remove(DropDownListTemp.SelectedItem);

                    }
                }
            }
            catch (IOException)
            {
                LabelError.Text = "ERROR. No se ha podido eliminar el archivo";
            }
            catch (SqlException)
            {
                LabelError.Text = "ERROR. No se ha podido eliminar el archivo";
            }
        }

        protected void btEliminarRef_Click(object sender, EventArgs e)
        {
            try
            {
                if ((DropDownListRef.Items.Count > 0) && (DropDownListRef.SelectedItem != null))
                {
                    Referencia refe = new Referencia(DropDownListRef.SelectedItem.Text, new Publicacion(pub.getID()));
                    refe.eliminar(); //Si no estaba en la BD no hace nada (no da error)
                    DropDownListRef.Items.Remove(DropDownListRef.SelectedItem.Text);
                    LabelCorrecto.Text = "La referencia ha sido eliminada con éxito.";
                }
                else
                {
                    LabelError.Text = "Debe seleccionar la referencia.";
                }
            }
            catch (SqlException)
            {
                LabelError.Text = "ERROR. No se ha podido eliminar la referencia.";
            }
        }

        protected void btAnadirArch_Click(object sender, EventArgs e)
        {
            try
            {
                string rutaTemp = Estatica.getRutaProyecto + "temp" + Path.DirectorySeparatorChar + idCurrentUser;
                if (!Directory.Exists(rutaTemp))
                {
                    Directory.CreateDirectory(rutaTemp);
                }

                if (FileUploadArch.HasFile && FileUploadArch.PostedFile.ContentLength < 10485760)
                {
                    if (FileUploadArch.FileName.ToLower().EndsWith(".jpg") ||
                        FileUploadArch.FileName.ToLower().EndsWith(".jpeg") ||
                        FileUploadArch.FileName.ToLower().EndsWith(".bmp") ||
                        FileUploadArch.FileName.ToLower().EndsWith(".gif") ||
                        FileUploadArch.FileName.ToLower().EndsWith(".tiff") ||
                        FileUploadArch.FileName.ToLower().EndsWith(".tif") ||
                        FileUploadArch.FileName.ToLower().EndsWith(".png"))
                    {
                        //Cambio los caracteres que al subir al servidor da error la URL
                        string nombre = FileUploadArch.FileName;
                        nombre = nombre.Replace('%', '-');
                        nombre = nombre.Replace('#', '-');
                        nombre = nombre.Replace('_', '-');
                        nombre = nombre.Replace(" ", "");

                        if (CheckBoxAsoc.Checked)
                        {
                            FileUploadArch.SaveAs(rutaTemp + Path.DirectorySeparatorChar + "red" + nombre);
                            redimensionar(rutaTemp + Path.DirectorySeparatorChar + "red" + nombre, rutaTemp + Path.DirectorySeparatorChar + nombre);
                            File.Delete(rutaTemp + Path.DirectorySeparatorChar + "red" + nombre);

                            //Si el archivo con ese nombre existia en la carpeta se sobreescribe y en ese caso ya estaba en el 
                            //dropdownlist asi que no lo vuelvo a agregar
                            if (DropDownListTemp.Items.FindByText(nombre) == null)
                            {
                                DropDownListTemp.Items.Add(nombre);
                            }
                        }
                        else
                        {
                            FileUploadArch.SaveAs(rutaTemp + Path.DirectorySeparatorChar + "red" + nombre);
                            redimensionar(rutaTemp + Path.DirectorySeparatorChar + "red" + nombre,
                                rutaTemp + Path.DirectorySeparatorChar + nombre);
                            File.Delete(rutaTemp + Path.DirectorySeparatorChar + "red" + nombre);

                            Archivo arch = new Archivo(rutaTemp + Path.DirectorySeparatorChar + nombre, new Usuario(idCurrentUser), new Publicacion(-1));
                            string nombreFinal = arch.almacenar();

                            //Si el archivo con ese nombre existia en la carpeta se sobreescribe y en ese caso ya estaba en el 
                            //dropdownlist asi que no lo vuelvo a agregar
                            if (DropDownListTemp.Items.FindByText(Estatica.getUrlProyecto + "Archivos/" + nombreFinal) == null)
                            {
                                DropDownListTemp.Items.Add(Estatica.getUrlProyecto + "Archivos/" + nombreFinal);
                            }
                        }
                    }
                    else if (FileUploadArch.FileName.ToLower().EndsWith(".wmv") ||
                        FileUploadArch.FileName.ToLower().EndsWith(".avi") ||
                        FileUploadArch.FileName.ToLower().EndsWith(".mov") ||
                        FileUploadArch.FileName.ToLower().EndsWith(".mpg") ||
                        FileUploadArch.FileName.ToLower().EndsWith(".mpeg") ||
                        FileUploadArch.FileName.ToLower().EndsWith(".mid") ||
                        FileUploadArch.FileName.ToLower().EndsWith(".midi") ||
                        FileUploadArch.FileName.ToLower().EndsWith(".mp3"))
                    {
                        string nombre = FileUploadArch.FileName;
                        nombre = nombre.Replace('%', '-');
                        nombre = nombre.Replace('#', '-');
                        nombre = nombre.Replace('_', '-');
                        nombre = nombre.Replace(" ", "");

                        //Si el archivo no debe ir asociado a una publicación lo reconozco pq empieza por a_
                        if (CheckBoxAsoc.Checked)
                        {
                            FileUploadArch.SaveAs(rutaTemp + Path.DirectorySeparatorChar + nombre);
                            if (DropDownListTemp.Items.FindByText(nombre) == null)
                                DropDownListTemp.Items.Add(nombre);
                        }
                        else
                        {
                            FileUploadArch.SaveAs(rutaTemp + Path.DirectorySeparatorChar + nombre);
                            Archivo arch = new Archivo(rutaTemp + Path.DirectorySeparatorChar + nombre, new Usuario(idCurrentUser), new Publicacion(-1));
                            string nombreFinal = arch.almacenar();

                            if (DropDownListTemp.Items.FindByText(Estatica.getUrlProyecto + "Archivos/" + nombreFinal) == null)
                                DropDownListTemp.Items.Add(Estatica.getUrlProyecto + "Archivos/" + nombreFinal);
                        }
                    }
                    else
                    {
                        LabelError.Text = "ERROR. Tipo de archivo no soportado";
                    }
                }
                else if (FileUploadArch.PostedFile.ContentLength > 10485760)
                {
                    LabelError.Text = "ERROR. El archivo no puede tener un tamaño superior a 10Mb";
                }
                else
                {
                    LabelError.Text = "ERROR. El archivo indicado no existe o no es accesible.";
                }
            }
            catch (IOException)
            {
                LabelError.Text = "Se ha producido un error al intentar guardar el archivo.";
            }
        }

        private void guardarArchivos(int idNot)
        {
            if (Directory.Exists(Estatica.getRutaProyecto + "temp" + Path.DirectorySeparatorChar + idCurrentUser))
            {
                string[] archs = Directory.GetFiles(Estatica.getRutaProyecto + "temp" + Path.DirectorySeparatorChar + idCurrentUser);
                if ((archs != null) && (archs.Length > 0))
                {
                    foreach (string arch in archs)
                    {
                        Archivo a = new Archivo(arch, new Usuario(idCurrentUser), new Publicacion(idNot));
                        a.almacenar();
                    }
                }
            }
        }

        protected void btAnadirRef_Click(object sender, EventArgs e)
        {
            if (TextBoxRef.Text.Equals(""))
            {
                LabelError.Text = "No puede añadir una referencia vacía";
            }
            else
            {
                DropDownListRef.Items.Add(TextBoxRef.Text);
                TextBoxRef.Text = "";
            }
        }

        public void redimensionar(string origen, string destino)
        {
            try
            {
                System.Drawing.Image image1 = new System.Drawing.Bitmap(origen);
                int destinoWidth = image1.Width;
                int destinoHeight = image1.Height;
                while ((destinoWidth > 700) || (destinoHeight > 700))
                {
                    //Las redimensiono un tanto por ciento para no perder las proporciones
                    destinoWidth = destinoWidth * 80 / 100;
                    destinoHeight = destinoHeight * 80 / 100;
                }

                Bitmap Imagen2 = new Bitmap(destinoWidth, destinoHeight);
                using (Graphics g = Graphics.FromImage((System.Drawing.Image)Imagen2))
                {
                    g.DrawImage(image1, 0, 0, destinoWidth, destinoHeight);
                }
                Imagen2.Save(destino);
                Imagen2.Dispose();
                image1.Dispose();
            }
            catch (IOException)
            {
                LabelError.Text = "ERROR. No se ha podido subir el archivo.";
            }
        }

        private void guardarReferencias(int idNot)
        {
            if (idNot == -1) { }//es que se ha producido un error
            else
            {
                for (int i = 0; i < DropDownListRef.Items.Count; i++)
                {
                    Referencia reff = new Referencia(DropDownListRef.Items[i].Text, new Publicacion(idNot));
                    if (!reff.existeEnBD())
                        reff.almacenar();
                }
            }
        }

        protected void btUsar_Click(object sender, EventArgs e)
        {

            if (TextBoxDescripcion.Visible)
            {
                TextBoxDescripcion.Text = TextBoxDescripcion.Text + DropDownListTemp.SelectedItem.Text;
            }
            else
            {
                FreeTextBox.Text = FreeTextBox.Text + DropDownListTemp.SelectedItem.Text;
            }
        }

    }
}
