﻿using System;
using System.Collections;
using System.Configuration;
using System.Data;
using System.Linq;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Xml.Linq;
using DivulgacionCientifica.CapaNegocio;
using System.Data.SqlClient;
using System.IO;

namespace DivulgacionCientifica
{
    public partial class GestionarTemas : System.Web.UI.Page
    {
        string idCurrentUser = HttpContext.Current.User.Identity.Name;
        protected void Page_Load(object sender, EventArgs e)
        {
            try
            {
                if (!Usuario.esAdministrador(idCurrentUser))
                {
                    Response.Redirect("~\\Inicio.aspx");
                }
                else
                {
                    //Añaden atributos al boton eliminar para que pida confirmación
                    btEliminar.Attributes.Add("language", "javascript");
                    btEliminar.Attributes.Add("OnClick", "return confirm('¿Esta seguro que desea eliminar el tema seleccionado? Recuerde que además"+
                        " se eliminarán las publicaciones que estén asociadas únicamente a ese tema.')");
                    if (!IsPostBack) //Para que solo se ejecute en la primera carga de la pagina y no cada vez que solicite
                    //algo al servidor y este lo devuelva (se pulse un boton...)
                    {
                        TextBoxURL.Text = "http://";
                    }
                    LabelCrear.Text = "";
                    LabelCrearCorrecto.Text = "";
                    LabelEli.Text = "";
                    LabelEliCorrecto.Text = "";
                    if (ListBoxTemas.Items.Count == 0)
                    {
                        Auxiliares.cargarTemas(ListBoxTemas);
                    }

                }
            }
            catch (SqlException)
            {
                LabelEli.Text = "ERROR. Se ha producido un error al cargar los temas existentes.";
            }
        }

        protected void btCrear_Click(object sender, EventArgs e)
        {
            Tema tema = new Tema(TextBoxNombre.Text, TextBoxDescripcion.Text);

            try
            {
                if (TextBoxNombre.Text.Length > 45)
                {
                    LabelCrear.Text = "El tema ya existe.";
                }
                else if (TextBoxDescripcion.Text.Length > 5000)
                {
                    LabelCrear.Text = "La descripcion del tema no puede exceder los 5000 caracteres.";
                }
                else if (TextBoxURL.Text.Length > 45)
                {
                    LabelCrear.Text = "La URL no puede exceder los 45 caracteres.";
                }
                else if (tema.existe())
                {
                    LabelCrear.Text = "ERROR. El nombre del tema ya está siendo usado.";
                }
                else if (tema.getNombre().Equals(""))
                {
                    LabelCrear.Text = "ERROR. El nombre del tema no puede ser vacio.";
                }
                else if (tema.getNombre().Contains("-"))
                {
                    LabelCrear.Text = "ERROR. El título del tema no puede contener guiones.";
                }
                else if (!urlValida(TextBoxURL.Text))
                {
                    LabelCrear.Text = "ERROR. Debe introducir una URL válida.";
                }
                else
                {
                    tema.almacenarTema();
                    ListBoxTemas.Items.Clear();
                    Auxiliares.cargarTemas(ListBoxTemas);

                    string ruta = Estatica.getRutaProyecto + "feedsRSS" + Path.DirectorySeparatorChar +
                        Path.DirectorySeparatorChar + tema.getNombre() + ".xml";
                    if (!System.IO.File.Exists(ruta))
                    {
                        tema.crearRSS(TextBoxURL.Text, ruta);
                    }
                    TextBoxDescripcion.Text = "";
                    LabelCrearCorrecto.Text = "El tema ha sido creado con éxito";
                    TextBoxNombre.Text = "";
                    TextBoxDescripcion.Text = "";
                    TextBoxURL.Text = "http://";
                }
            }
            catch (SqlException)
            {
                LabelCrear.Text = "ERROR. Se ha producido un error al almacenar el tema en la base de datos";
            }
            catch (IOException ex)
            {
                LabelCrear.Text = "ERROR. Se ha producido un error asociado al feed:" + ex.ToString()+". El tema NO se ha almacenado.";
                //Si se produce un error aqui no se crea el archivo RSS correctamente por lo que debo
                //eliminar el tema de la base de datos
                tema.eliminarTemaFisico();
            }

        }

        protected void btEliminar_Click(object sender, EventArgs e)
        {
            try
            {
                Tema tema = new Tema(ListBoxTemas.SelectedItem.ToString(), ""); //La descripcion no me importa para borrarlo
                tema.eliminarTemaLogico();
                LabelEliCorrecto.Text = "El tema ha sido eliminado con éxito";
                ListBoxTemas.Items.Clear();
                Auxiliares.cargarTemas(ListBoxTemas);
            }
            catch (AccessViolationException)//(NullReferenceException)
            {
                LabelEli.Text = "ERROR. Debe seleccionar un tema";
            }
            catch (SqlException)
            {
                LabelEli.Text = "ERROR. Ha ocurrido un error y puede que el tema no haya sido eliminado.";
            }

        }

        public static bool urlValida(string url)
        {
            return url.StartsWith("http://") && url.Contains(".") && !url.Contains(" ");
        }
      
    }
}
