﻿using System;
using System.Collections;
using System.Configuration;
using System.Data;
using System.Linq;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Xml.Linq;
using System.Data.SqlClient;
using System.Collections.Generic;
using System.IO;
using DivulgacionCientifica.CapaNegocio;
using System.Net.Mail;

namespace DivulgacionCientifica
{
    public partial class PetEditPublic : System.Web.UI.Page
    {
        string idCurrentUser = HttpContext.Current.User.Identity.Name;
        Publicacion publ;
        EditPublic modPubl;

        protected void Page_Load(object sender, EventArgs e)
        {
            if (!Usuario.esAdministrador(idCurrentUser))
                Response.Redirect("Inicio.aspx");

            lBtRechazar.Attributes.Add("language", "javascript");
            lBtRechazar.Attributes.Add("OnClick", "return confirm('¿Esta seguro que desea RECHAZAR la petición del usuario para modificar esta publicación?')");

            lBtValidar.Attributes.Add("language", "javascript");
            lBtValidar.Attributes.Add("OnClick", "return confirm('¿Esta seguro que desea VALIDAR la petición del usuario para modificar esta publicación?')");

            int id = Int32.Parse(Request.Params["id"]);
                
            try
            {
                publ = new Publicacion(id);
                modPubl = new EditPublic(id);

                if (!IsPostBack)
                {
                    LabelCategoria.Text = modPubl.getCategoria();
                    LabelCategAnt.Text = publ.getCategoria();
                    LabelTitulo.Text = modPubl.getTitulo();
                    LabelTituloAnt.Text = publ.getTitulo();
                    LabelCaducidad.Text = modPubl.getCaducidad();
                    if (publ.getCaducidad().Equals("NULL")) { LabelCaducAnt.Text = "No tiene."; } 
                    else { LabelCaducAnt.Text = Auxiliares.StringAFecha(publ.getCaducidad()).ToString().Substring(0, 10); }
                    PanelContenido.Controls.Add(new LiteralControl(Auxiliares.textoPlanoAHtml(modPubl.getContenido())));
                    PanelContenidoAnt.Controls.Add(new LiteralControl(Auxiliares.textoPlanoAHtml(publ.getContenido())));
                    Auxiliares.cargarArchivos(PanelArchivos, modPubl.archivosNoValidados());
                    Auxiliares.cargarArchivos(PanelArchivosAnt, publ.getArchivos(false));
                    Auxiliares.cargarReferencias(PanelReferencias, modPubl.getReferenciasNoValidadas());
                    Auxiliares.cargarReferencias(PanelReferenciasAnt, publ.getReferencias(true));
                    if (modPubl.getMotivo().Equals(""))
                    {
                        PanelMotivo.Controls.Add(new LiteralControl("El usuario no expuso el motivo por el cual desea modificar su publicación."));
                    }
                    else
                    {
                        PanelMotivo.Controls.Add(new LiteralControl(Auxiliares.textoPlanoAHtml(modPubl.getMotivo())));
                    }
                }
            }
            catch (SqlException)
            {
                LabelError.Text = "ERROR. No se ha podido cargar la petición.";
            }
            catch (FormatException)
            {
                //Se lanza cuando id es incorrecto.
                LabelError.Text = "ERROR. No se reconoce la publicación.";
            }
        }

        protected void lBtRechazar_Click(object sender, EventArgs e)
        {
            try
            {
                modPubl.eliminar();
                publ.eliminarArchivosNoValidados();
                publ.eliminarReferenciasNoValidadas();
                Response.Redirect("Peticiones.aspx");
            }
            catch (SqlException)
            {
                LabelError.Text = "ERROR. No se ha podido rechazar la petición.";
            }
        }

        private void enviarEmailAfirm()
        {
            try
            {
                MailMessage correo = new MailMessage();
                correo.From = new System.Net.Mail.MailAddress(Estatica.getMailAdress, "Divulgacion Científica", System.Text.Encoding.UTF8);
                correo.To.Add(publ.getCreador().getEmail());
                correo.Subject = "Publicación modificada";
                correo.Body = "Hola " + publ.getCreador().getNombre() + @",

Le comunicamos que ha sido aceptada su petición para modificar la publicaión con título: "+publ.getTitulo()+@".

Recuerde que puede verla en " + Estatica.getUrlProyecto + "VisualizarPublicacion.aspx?id=" + publ.getID() + @"

Gracias.";
                correo.IsBodyHtml = false;
                correo.Priority = System.Net.Mail.MailPriority.Normal;

                SmtpClient smtp = new SmtpClient();
                smtp.Host = Estatica.getSmtp;
                smtp.EnableSsl = Estatica.getSmtpUsaSSL;
                if (Estatica.getSmtpUsaSSL)
                {
                    smtp.Credentials = new System.Net.NetworkCredential(Estatica.getMailAdress, Estatica.getMailPassw);
                }
                smtp.Send(correo);
            }
            catch (FormatException)
            {
                LabelError.Text = "AVISO. No se ha podido enviar el email al creador debido ya que su dirección no tiene el formato correcto.";
            }
            catch (SmtpException)
            {
                LabelError.Text = "AVISO. No se ha podido enviar el email al creador.";
            }
        }

        private void enviarEmailNegat()
        {
            try
            {
                MailMessage correo = new MailMessage();
                correo.From = new System.Net.Mail.MailAddress(Estatica.getMailAdress, "Divulgacion Científica", System.Text.Encoding.UTF8);
                correo.To.Add(publ.getCreador().getEmail());
                correo.Subject = "Publicación no modificada";
                correo.Body = "Hola " + publ.getCreador().getNombre() + @",

Le informamos que su petición para modificar la publicación con título "+publ.getTitulo()+@" ha sido rechazada.
                
Lo sentimos.";

                correo.IsBodyHtml = false;
                correo.Priority = System.Net.Mail.MailPriority.Normal;

                SmtpClient smtp = new SmtpClient();
                smtp.Host = Estatica.getSmtp;
                smtp.EnableSsl = Estatica.getSmtpUsaSSL;
                if (Estatica.getSmtpUsaSSL)
                {
                    smtp.Credentials = new System.Net.NetworkCredential(Estatica.getMailAdress, Estatica.getMailPassw);
                }
                smtp.Send(correo);
            }
            catch (FormatException)
            {
                LabelError.Text = "AVISO. No se ha podido enviar el email al creador debido ya que su direccion no tiene el formato correcto.";
            }
            catch (SmtpException)
            {
                LabelError.Text = "AVISO. No se ha podido enviar el email al creador.";
            }
        }

        protected void lBtValidar_Click(object sender, EventArgs e)
        {
            try
            {
                EditPublic edt = new EditPublic(Int32.Parse(Request.Params["id"]));
                try
                {
                    edt.eliminar();
                }
                catch (SqlException)
                {
                    LabelError.Text = "ERROR. No se han podido validar la petición.";
                }
                Publicacion p = new Publicacion(Int32.Parse(Request.Params["id"]));
                /**Hay que preparar la fecha para MySQL***/
                string fe;
                if (edt.getCaducidad().Equals("No tiene.") || edt.getCaducidad().Equals("NULL"))
                {
                    fe = "NULL";
                }
                else
                {
                    DateTime fecha = DateTime.Parse(edt.getCaducidad());
                    fe = "'" + fecha.Year.ToString() + "-" + fecha.Month.ToString() + "-" +
                        fecha.Day.ToString() + " " + fecha.Hour.ToString() +
                        ":" + fecha.Minute.ToString() + ":" + fecha.Second.ToString() + "'";
                }
                /*****/
                p.modificarCaducidad(fe);
                p.modificarCategoria(edt.getCategoria());
                p.modificarContenido(edt.getContenido());
                p.modificarTitulo(edt.getTitulo());
                p.validarReferencias();
                p.validarArchivos();
                enviarEmailAfirm();
                Response.Redirect("Peticiones.aspx");
            }
            catch (SqlException)
            {
                LabelError.Text = "ERROR. No se han podido modificar los datos";
            }
            catch (NullReferenceException)
            {
                LabelError.Text = "ERROR. La petición ya no se encuentra disponible.";
            }
        }

        
    }
}
